from string import Template
from stravacookies import StravaCookieFetcher

class OnlineMap(object):
# Constants:
## Heatmap colors
    COLORS = {  "hot": "hot",
                "blue": "blue",
                "purple": "purple",
                "gray": "gray",
                "red": "bluered"}

## Heatmap activities
    ACTIVITIES = {  "all": "all",
                    "ride": "ride",
                    "run": "run",
                    "winter": "winter"}

## Names that will be given to heatmaps based on their activity
    MAPNAMES = {"all": "Strava Heatmap (all)",
                "ride": "Strava Heatmap (ride)",
                "run": "Strava Heatmap (run)",
                "winter": "Strava Heatmap (winter)"}

## Heatmap URLs will be generated from the following template,
## replacing placeholders 'activity', 'color', and 'cookieString' with actual values
    URL_TEMPLATE = Template("https://heatmap-external-a.strava.com/tiles-auth/$activity/$color/{z}/{x}/{y}.png?$cookieString")

    def getDefinition(heatmapColor, stravaEmail, stravaPassword):
# Returns a python dictionary representing an online map definition;
# dumping that dictionary to a json file results in an
# online map definition file that can be imported into Cartograph Maps
        stravaCookieFetcher = StravaCookieFetcher()
        stravaCookieFetcher.fetchCookies(stravaEmail, stravaPassword)
        cookieString = stravaCookieFetcher.getCookieString()

        maps = []

        for key in OnlineMap.ACTIVITIES:
            url = OnlineMap.URL_TEMPLATE.substitute(activity = OnlineMap.ACTIVITIES[key], color = OnlineMap.COLORS[heatmapColor], cookieString = cookieString)

            map = {
                "name": OnlineMap.MAPNAMES[key],
                "type": "ONLINE", # Possible values: "ONLINE" or "WMS"
                "url": url,
                "attribution": "© Strava",
                "description": "",
                "defaultLatitude": 45.0781,
                "defaultLongitude": 7.6761,
                "defaultZoom": 11,
                "minZoom": 2,
                "maxZoom": 22,
                "projection": "EPSG_4326", # Possible values: "EPSG_4326" (default) or "EPSG_900913"
                "headers": [
                    {
                    "key": "User-Agent",
                    "value": "Cartograph"
                    }
                ]
            }
            maps.append(map)

        onlineMapDef = {
            "version": 2,
            "maps": maps
        }

        return onlineMapDef



# -*- coding: utf-8 -*-

import json
import platform
import sys

from pathlib import Path

from colorama import Fore, Style
from getpass import getpass

from .onlinemap import OnlineMap
from stravacookies import StravaCFetchCookieError

# Set location where output file will be written
outdir = "./"
print("If you are on MacOS, you can save the map file directly in the Cartograph folder")
print("in iCloud, otherwise it will be saved in the current working directory.")
while True:
    print("Do you want to save the map file in iCloud (y/N)?")
    answer = input()
    if (answer == "y" or answer == "Y"):
        outdir = str(Path.home()) + "/Library/Mobile Documents/com~apple~CloudDocs/Cartograph Pro/"
        break
    elif (answer == "n" or answer == "N" or answer == ""):
        # leave outdir unchanged
        break


print("What map color do you want?")
for color in OnlineMap.COLORS:
    print ("  " + color)
while True:
    print("?")
    answer = input()
    if answer in OnlineMap.COLORS:
        break
color = answer

stravaEmail = input(f"{Fore.CYAN}**** Enter your Strava account email:{Style.RESET_ALL} ")
stravaPassword = getpass(f"{Fore.CYAN}**** Enter your Strava account password:{Style.RESET_ALL} ")

try:
    print("Creating online map definition file... ", end="", flush=True)

    onlineMapDef = OnlineMap.getDefinition(color, stravaEmail, stravaPassword)

    print(json.dumps(onlineMapDef), file=open(outdir + "carto_strava.onlinemap", "w"))
    print("done.")

except StravaCFetchCookieError as e:
    print("Strava cookies unavailable.", file=sys.stderr)
    print(e.message, file=sys.stderr)
except Exception as e:
    print("An error has occurred: " + e.message, file=sys.stderr)



import os
import datetime
from shutil import copy
import xml.etree.ElementTree as ET

from stravacookies.fetcher import StravaCookieFetcher

class ImgUpdater(object):
    def __init__(self):
        self.josmPreferences = "./fakepath.xml" # has to be set in OS-specific constructor
        self.cookieFetcher=StravaCookieFetcher() # may be overridden in OS-specific constructor

    def bakPrefs(self):
        bakfile = self.josmPreferences + '.' + datetime.datetime.now().strftime("%Y%m%d%H%M%S") + '.bak'
        copy(self.josmPreferences, bakfile)

    def updPrefs(self, stravaEmail, stravaPassword):
        stravaCookieFetcher = self.cookieFetcher
        stravaCookieFetcher.fetchCookies(stravaEmail, stravaPassword)
        cookieString = stravaCookieFetcher.getCookieString()
        ET.register_namespace('', "http://josm.openstreetmap.de/preferences-1.0")
        doc = ET.parse(self.josmPreferences)
        root = doc.getroot()
        for maptag in doc.findall("./{http://josm.openstreetmap.de/preferences-1.0}maps/{http://josm.openstreetmap.de/preferences-1.0}map/{http://josm.openstreetmap.de/preferences-1.0}tag[@key='url']"):
            if "strava.com" in maptag.attrib["value"]:
                print("Updating Strava Imagery URL...")
                fullUrl = maptag.attrib["value"]
                splittedUrl = fullUrl.split('?')
                baseUrl = splittedUrl[0]
                print("--> [" + baseUrl + "]")
                fullUrl = baseUrl + "?" + cookieString
                maptag.attrib["value"] = fullUrl
        print("Writing out preferences.xml...")
        doc.write(self.josmPreferences, encoding="UTF-8")


class MacOsImgUpdater(ImgUpdater):
    def __init__( self ):
        super().__init__()
        self.josmPreferences = os.path.expanduser('~/Library/Preferences/JOSM/preferences.xml')


class LinuxImgUpdater(ImgUpdater):
    def __init__( self ):
        super().__init__()
        # On Linux, three possible locations for JOSM preference file
        # according to https://josm.openstreetmap.de/wiki/Help/Preferences
        # (plus, users may specify a custom directory within JOSM;
        # this script doesn't support that)
        pref = os.path.expanduser('~/.josm/preferences.xml')
        if not os.path.isfile( pref ):
            try:
                pref = os.path.join(
                    os.environ['XDG_CONFIG_HOME'],
                    'JOSM', 'preferences.xml')
            except KeyError:
                pref = os.path.expanduser('~/.config/JOSM/preferences.xml')
        if not os.path.isfile( pref ):
            message = "Supported locations of preferences.xml:\n"
            message+= "~/.josm/, $XDG_CONFIG_HOME/JOSM, or ~/.config/JOSM\n"
            message+= "(custom preference locations are not supported)"
            raise StravaCFetchJosmprefsError(message)
        self.josmPreferences = pref


class WindowsImgUpdater(ImgUpdater):
    def __init__( self ):
        super().__init__()
        self.josmPreferences = os.path.join(
            os.path.expandvars('%APPDATA%\JOSM'),
            'preferences.xml')
        # location of JOSM preference file for Windows
        # according to https://josm.openstreetmap.de/wiki/Help/Preferences
        if not os.path.isfile( self.josmPreferences ):
            message = "Expect preferences.xml to sit in %APPDATA%\JOSM"
            raise StravaCFetchJosmprefsError(message)



import platform
import sys

from colorama import Fore, Style
from getpass import getpass

from stravacookies import ( StravaCFetchCookieError,
                            StravaCFetchOsError,
                            StravaCFetchJosmprefsError
                          )
from .imgupdater import    ( ImgUpdater,
                            MacOsImgUpdater,
                            LinuxImgUpdater,
                            WindowsImgUpdater
                          )

try:
    if (platform.system() == "Darwin"):
        stravaImgUpdater = MacOsImgUpdater()
    elif (platform.system() == "Linux"):
        stravaImgUpdater = LinuxImgUpdater()
    elif (platform.system() == "Windows"):
        stravaImgUpdater = WindowsImgUpdater()
    else:
        raise StravaCFetchOsError(platform.system())

    stravaEmail = input(f"{Fore.CYAN}**** Enter your Strava account email:{Style.RESET_ALL} ")
    stravaPassword = getpass(f"{Fore.CYAN}**** Enter your Strava account password:{Style.RESET_ALL} ")

    print("Backing up preferences.xml...")
    stravaImgUpdater.bakPrefs()
    print("Updating preferences.xml...")
    stravaImgUpdater.updPrefs(stravaEmail, stravaPassword)
    print("Done.")

except StravaCFetchOsError as e:
    print("Only macOS/linux/Windows are supported.", file=sys.stderr)
    print("Detected OS: " + e.message, file=sys.stderr)
except StravaCFetchCookieError as e:
    print("Strava cookies unavailable.", file=sys.stderr)
    print(e.message, file=sys.stderr)
except StravaCFetchJosmprefsError as e:
    print("Couldn't locate JOSM preferences file.", file=sys.stderr)
    print( e.message, file=sys.stderr)
